@@ -77,15 +77,31 @@ impl<'a, T: 'a + Copy + Ord, I: IntoIterator<Item = &'a T>, const N: usize> Exte
   }
 }
 
+impl<T: Ord, const N1: usize, const N2: usize> ExtendEx<T, StaticHeap<T, N1>>
+  for StaticHeap<T, N2>
+{
+  #[inline(always)]
+  default fn extend_ex(&mut self, other: StaticHeap<T, N1>) {
+    self.data.extend(other.data);
+    self.rebuild();
+  }
+
+  #[inline(always)]
+  default fn from_iter_ex(iter: StaticHeap<T, N1>) -> Self {
+    Self::from(iter.data)
+  }
+}
+
 impl<T: Ord, const N: usize> ExtendEx<T, StaticHeap<T, N>> for StaticHeap<T, N> {
   #[inline(always)]
-  fn extend_ex(&mut self, ref mut other: StaticHeap<T, N>) {
-    self.append(other);
+  fn extend_ex(&mut self, other: StaticHeap<T, N>) {
+    self.data.extend(other.data);
+    self.rebuild();
   }
 
   #[inline(always)]
   fn from_iter_ex(iter: StaticHeap<T, N>) -> Self {
-    StaticHeap::from(iter.into_iter().collect::<StaticVec<_, N>>())
+    Self::from(iter.data)
   }
 }
 
@@ -104,12 +120,15 @@ impl<'a, T: 'a + Copy + Ord, const N: usize> Extend<&'a T> for StaticHeap<T, N>
 }
 
 impl<T: Ord, const N1: usize, const N2: usize> From<StaticVec<T, N1>> for StaticHeap<T, N2> {
-  /// Converts a `StaticVec<T, N>` into a `StaticHeap<T, N>`.
+  /// Converts a `StaticVec<T, N1>` into a `StaticHeap<T, N2>`.
   /// This conversion happens in-place, and has `O(n)` time complexity.
   #[inline(always)]
   default fn from(vec: StaticVec<T, N1>) -> StaticHeap<T, N2> {
     let mut heap = StaticHeap {
-      data: StaticVec::from_iter(vec.into_iter()),
+      // We take advantage of one of StaticVec's `FromIter` specializations here, which
+      // basically works like a `From<StaticVec<T, N1>> for StaticVec<T, N2>` impl would if
+      // it could be implemented directly.
+      data: StaticVec::from_iter(vec),
     };
     heap.rebuild();
     heap
@@ -127,6 +146,32 @@ impl<T: Ord, const N: usize> From<StaticVec<T, N>> for StaticHeap<T, N> {
   }
 }
 
+impl<T: Ord, const N1: usize, const N2: usize> From<[T; N1]> for StaticHeap<T, N2> {
+  /// Converts a `[T; N1]` into a `StaticHeap<T, N2>`.
+  /// This conversion happens in-place, and has `O(n)` time complexity.
+  #[inline(always)]
+  default fn from(array: [T; N1]) -> StaticHeap<T, N2> {
+    let mut heap = StaticHeap {
+      data: StaticVec::new_from_array(array),
+    };
+    heap.rebuild();
+    heap
+  }
+}
+
+impl<T: Ord, const N: usize> From<[T; N]> for StaticHeap<T, N> {
+  /// Converts a `[T; N]` into a `StaticHeap<T, N>`.
+  /// This conversion happens in-place, and has `O(n)` time complexity.
+  #[inline(always)]
+  fn from(array: [T; N]) -> StaticHeap<T, N> {
+    let mut heap = StaticHeap {
+      data: StaticVec::new_from_const_array(array),
+    };
+    heap.rebuild();
+    heap
+  }
+}
+
 impl<T: Ord, const N: usize> FromIterator<T> for StaticHeap<T, N> {
   #[inline(always)]
   fn from_iter<I: IntoIterator<Item = T>>(iter: I) -> StaticHeap<T, N> {
