@@ -1,4 +1,3 @@
-use core::intrinsics::size_of;
 use core::mem::swap;
 
 use self::heap_helpers::StaticHeapHole;
@@ -15,7 +14,7 @@ mod heap_trait_impls;
 ///
 /// `StaticHeap`, as well as the associated iterator and helper structs for it are direct
 /// adaptations of the ones found in the `std::collections::binary_heap` module (including
-/// most of the documentation).
+/// most of the documentation, at least for the functions that exist in both implementations).
 ///
 /// It is a logic error for an item to be modified in such a way that the
 /// item's ordering relative to any other item, as determined by the `Ord`
@@ -70,11 +69,11 @@ mod heap_trait_impls;
 /// value instead of the greatest one.
 ///
 /// ```
-/// use staticvec::{staticvec, StaticHeap};
+/// use staticvec::StaticHeap;
 /// use core::cmp::Reverse;
 ///
 /// // Wrap the values in `Reverse`.
-/// let mut heap = StaticHeap::from(staticvec![Reverse(1), Reverse(5), Reverse(2)]);
+/// let mut heap = StaticHeap::from([Reverse(1), Reverse(5), Reverse(2)]);
 ///
 /// // If we pop these scores now, they should come back in the reverse order.
 /// assert_eq!(heap.pop(), Some(Reverse(1)));
@@ -108,7 +107,7 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::StaticHeap;
-  /// let mut heap = StaticHeap::<i32, 12>::new();
+  /// let mut heap = StaticHeap::<i32, 2>::new();
   /// heap.push(4);
   /// ```
   #[inline(always)]
@@ -129,7 +128,7 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::StaticHeap;
-  /// let mut heap = StaticHeap::<i32, 12>::new();
+  /// let mut heap = StaticHeap::<i32, 4>::new();
   /// assert!(heap.peek_mut().is_none());
   /// heap.push(1);
   /// heap.push(5);
@@ -156,6 +155,40 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
     }
   }
 
+  /// Pops a value from the end of the StaticHeap and returns it directly without asserting that
+  /// the StaticHeap's current length is greater than 0.
+  ///
+  /// # Safety
+  ///
+  /// It is up to the caller to ensure that the StaticHeap contains at least one
+  /// element prior to using this function. Failure to do so will result in reading
+  /// from uninitialized memory.
+  ///
+  /// # Examples
+  ///
+  /// Basic usage:
+  /// ```
+  /// # use staticvec::*;
+  /// let mut heap = StaticHeap::from([1, 3]);
+  /// unsafe {
+  ///   assert_eq!(heap.pop_unchecked(), 3);
+  ///   assert_eq!(heap.pop_unchecked(), 1);
+  /// }
+  /// ```
+  ///
+  /// # Time complexity
+  ///
+  /// The worst case cost of `pop_unchecked` on a heap containing *n* elements is O(log n).
+  #[inline(always)]
+  pub unsafe fn pop_unchecked(&mut self) -> T {
+    let mut res = self.data.pop_unchecked();
+    if self.is_not_empty() {
+      swap(&mut res, self.data.get_unchecked_mut(0));
+      self.sift_down_to_bottom(0);
+    }
+    res
+  }
+
   /// Removes the greatest item from the StaticHeap and returns it, or `None` if it
   /// is empty.
   ///
@@ -164,7 +197,7 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let mut heap = StaticHeap::from(staticvec![1, 3]);
+  /// let mut heap = StaticHeap::from([1, 3]);
   /// assert_eq!(heap.pop(), Some(3));
   /// assert_eq!(heap.pop(), Some(1));
   /// assert_eq!(heap.pop(), None);
@@ -175,13 +208,55 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// The worst case cost of `pop` on a heap containing *n* elements is O(log n).
   #[inline(always)]
   pub fn pop(&mut self) -> Option<T> {
-    self.data.pop().map(|mut item| {
-      if self.is_not_empty() {
-        swap(&mut item, unsafe { self.data.get_unchecked_mut(0) });
-        self.sift_down_to_bottom(0);
-      }
-      item
-    })
+    if self.is_empty() {
+      None
+    } else {
+      Some(unsafe { self.pop_unchecked() })
+    }
+  }
+
+  /// Pushes a value onto the StaticHeap without asserting that
+  /// its current length is less than `self.capacity()`.
+  ///
+  /// # Safety
+  ///
+  /// It is up to the caller to ensure that the length of the StaticHeap
+  /// prior to using this function is less than `self.capacity()`.
+  /// Failure to do so will result in writing to an out-of-bounds memory region.
+  ///
+  /// # Examples
+  ///
+  /// Basic usage:
+  /// ```
+  /// # use staticvec::StaticHeap;
+  /// let mut heap = StaticHeap::<i32, 3>::new();
+  /// unsafe {
+  ///   heap.push_unchecked(3);
+  ///   heap.push_unchecked(5);
+  ///   heap.push_unchecked(1);
+  /// }
+  /// assert_eq!(heap.len(), 3);
+  /// assert_eq!(heap.peek(), Some(&5));
+  /// ```
+  ///
+  /// # Time complexity
+  ///
+  /// The expected cost of `push_unchecked`, averaged over every possible ordering of
+  /// the elements being pushed, and over a sufficiently large number of
+  /// pushes, is O(1). This is the most meaningful cost metric when pushing
+  /// elements that are *not* already in any sorted pattern.
+  ///
+  /// The time complexity degrades if elements are pushed in predominantly
+  /// ascending order. In the worst case, elements are pushed in ascending
+  /// sorted order and the amortized cost per push is O(log n) against a heap
+  /// containing *n* elements.
+  ///
+  /// The worst case cost of a *single* call to `push_unchecked` is O(n).
+  #[inline(always)]
+  pub unsafe fn push_unchecked(&mut self, item: T) {
+    let old_length = self.len();
+    self.data.push_unchecked(item);
+    self.sift_up(0, old_length);
   }
 
   /// Pushes an item onto the StaticHeap, panicking if the underlying StaticVec
@@ -192,7 +267,7 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::StaticHeap;
-  /// let mut heap = StaticHeap::<i32, 12>::new();
+  /// let mut heap = StaticHeap::<i32, 5>::new();
   /// heap.push(3);
   /// heap.push(5);
   /// heap.push(1);
@@ -215,9 +290,12 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// The worst case cost of a *single* call to `push` is O(n).
   #[inline(always)]
   pub fn push(&mut self, item: T) {
-    let old_len = self.len();
+    // Deferring to our own `push_unchecked` which defers to `StaticVec::push_unchecked`
+    // is slower here than just calling `StaticVec::push` which calls `StaticVec::push_unchecked`
+    // anyways.
+    let old_length = self.len();
     self.data.push(item);
-    self.sift_up(0, old_len);
+    self.sift_up(0, old_length);
   }
 
   /// Consumes the StaticHeap and returns a StaticVec in sorted (ascending) order.
@@ -227,7 +305,7 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let mut heap = StaticHeap::from(StaticVec::<i32, 8>::from([1, 2, 4, 5, 7]));
+  /// let mut heap = StaticHeap::<i32, 8>::from([1, 2, 4, 5, 7]);
   /// heap.push(6);
   /// heap.push(3);
   /// let vec = heap.into_sorted_staticvec();
@@ -291,12 +369,6 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
     }
   }
 
-  #[inline(always)]
-  fn sift_down(&mut self, position: usize) {
-    let len = self.len();
-    self.sift_down_range(position, len);
-  }
-
   /// Takes an element from `position` and moves it all the way down the heap,
   /// then sifts it up to its position.
   ///
@@ -328,61 +400,37 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
     let mut n = self.len() / 2;
     while n > 0 {
       n -= 1;
-      self.sift_down(n);
+      self.sift_down_range(n, self.len());
     }
   }
 
-  /// Moves all the elements of `other` into `self`, leaving `other` empty.
+  /// Appends `self.remaining_capacity()` (or as many as available) items from `other` to `self`.
+  /// The appended items (if any) will no longer exist in `other` afterwards (which is to say,
+  /// `other` will be left empty.)
+  ///
+  /// The `N2` parameter does not need to be provided explicitly, and can be inferred directly from
+  /// the constant `N2` constraint of `other` (which may or may not be the same as the `N`
+  /// constraint of `self`.)
   ///
   /// # Examples
   ///
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let v = staticvec![-10, 1, 2];
-  /// let mut a = StaticHeap::<i32, 6>::from(v);
-  /// let v = staticvec![-20, 5, 43];
-  /// let mut b = StaticHeap::from(v);
+  /// // We give the two heaps arbitrary capacities for the sake of the example.
+  /// let mut a = StaticHeap::<i32, 9>::from([-10, 1, 2, 3, 3]);
+  /// let mut b = StaticHeap::<i32, 18>::from([-20, 5, 43]);
   /// a.append(&mut b);
-  /// assert_eq!(a.into_sorted_staticvec(), [-20, -10, 1, 2, 5, 43]);
+  /// assert_eq!(a.into_sorted_staticvec(), [-20, -10, 1, 2, 3, 3, 5, 43]);
   /// assert!(b.is_empty());
   /// ```
-  #[inline]
-  pub fn append(&mut self, other: &mut Self) {
-    // TODO: Investigate further whether the below calculations are
-    // actually applicable the same way they are for normal `BinaryHeap`,
-    // as our `extend` and `append` implementations are somewhat different
-    // (we have more specializations of `extend`, for example).
-
-    if self.len() < other.len() {
-      swap(self, other);
-    }
-
+  #[inline(always)]
+  pub fn append<const N2: usize>(&mut self, other: &mut StaticHeap<T, N2>) {
     if other.is_empty() {
       return;
     }
-
-    #[inline(always)]
-    fn log2_fast(x: usize) -> usize {
-      8 * size_of::<usize>() - (x.leading_zeros() as usize) - 1
-    }
-
-    // `rebuild` takes O(len1 + len2) operations
-    // and about 2 * (len1 + len2) comparisons in the worst case
-    // while `extend` takes O(len2 * log_2(len1)) operations
-    // and about 1 * len2 * log_2(len1) comparisons in the worst case,
-    // assuming len1 >= len2.
-    #[inline]
-    fn better_to_rebuild(len1: usize, len2: usize) -> bool {
-      2 * (len1 + len2) < len2 * log2_fast(len1)
-    }
-
-    if better_to_rebuild(self.len(), other.len()) {
-      self.data.append(&mut other.data);
-      self.rebuild();
-    } else {
-      self.extend(other.drain());
-    }
+    self.data.append(&mut other.data);
+    self.rebuild();
   }
 
   /// Returns an iterator which retrieves elements in heap order.
@@ -398,7 +446,7 @@ impl<T: Ord, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let mut heap = StaticHeap::from(staticvec![1, 2, 3, 4, 5]);
+  /// let mut heap = StaticHeap::from([1, 2, 3, 4, 5]);
   /// assert_eq!(heap.len(), 5);
   /// drop(heap.drain_sorted()); // removes all elements in heap order
   /// assert_eq!(heap.len(), 0);
@@ -437,7 +485,7 @@ impl<T, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let heap = StaticHeap::from(staticvec![1, 2, 3, 4, 5]);
+  /// let heap = StaticHeap::from([1, 2, 3, 4, 5]);
   /// assert_eq!(
   ///   heap.into_iter_sorted().take(2).collect::<StaticVec<_, 3>>(), staticvec![5, 4]
   /// );
@@ -454,7 +502,7 @@ impl<T, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let mut heap = StaticHeap::<i32, 12>::new();
+  /// let mut heap = StaticHeap::<i32, 7>::new();
   /// assert_eq!(heap.peek(), None);
   /// heap.push(1);
   /// heap.push(5);
@@ -513,13 +561,13 @@ impl<T, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let x = StaticHeap::<u8, 8>::from(staticvec![1, 2, 3, 4, 5, 6, 7, 8]);
+  /// let x = StaticHeap::<u8, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
   /// assert_eq!(x.size_in_bytes(), 8);
-  /// let y = StaticHeap::<u16, 8>::from(staticvec![1, 2, 3, 4, 5, 6, 7, 8]);
+  /// let y = StaticHeap::<u16, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
   /// assert_eq!(y.size_in_bytes(), 16);
-  /// let z = StaticHeap::<u32, 8>::from(staticvec![1, 2, 3, 4, 5, 6, 7, 8]);
+  /// let z = StaticHeap::<u32, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
   /// assert_eq!(z.size_in_bytes(), 32);
-  /// let w = StaticHeap::<u64, 8>::from(staticvec![1, 2, 3, 4, 5, 6, 7, 8]);
+  /// let w = StaticHeap::<u64, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
   /// assert_eq!(w.size_in_bytes(), 64);
   /// ```
   #[inline(always)]
@@ -569,7 +617,7 @@ impl<T, const N: usize> StaticHeap<T, N> {
   /// Basic usage:
   /// ```
   /// # use staticvec::*;
-  /// let mut heap = StaticHeap::<i32, 12>::new();
+  /// let mut heap = StaticHeap::<i32, 28>::new();
   /// assert!(heap.is_empty());
   /// ```
   #[inline(always)]
@@ -588,6 +636,10 @@ impl<T, const N: usize> StaticHeap<T, N> {
   /// heap.push(1);
   /// assert!(heap.is_not_empty());
   /// ```
+  // Clippy wants `!is_empty()` for this, but I prefer it as-is. My question is though, does it
+  // actually know that we have an applicable `is_empty()` function, or is it just guessing? I'm not
+  // sure.
+  #[allow(clippy::len_zero)]
   #[inline(always)]
   pub const fn is_not_empty(&self) -> bool {
     self.len() > 0
@@ -665,6 +717,6 @@ impl<T, const N: usize> StaticHeap<T, N> {
   /// ```
   #[inline(always)]
   pub fn clear(&mut self) {
-    self.drain();
+    self.data.clear();
   }
 }
