@@ -1,8 +1,15 @@
 #![no_std]
 #![allow(
+  // Clippy wants every single instance of the word "StaticVec" to be in syntax-highlight
+  // backticks, which IMO looks way too "noisy" when actually rendered.
   clippy::doc_markdown,
+  // Clippy thinks inline always is a bad idea even for the most simple of one-liners, so
+  // IMO it's just not a particularly helpful lint.
   clippy::inline_always,
+  // The "if-let" syntax Clippy recommends as an alternative to "match" in this lint is
+  // generally way less readable IMO.
   clippy::match_bool,
+  // Without this, every single use of const generics is warned against.
   incomplete_features
 )]
 #![feature(
@@ -17,6 +24,7 @@
   const_ptr_offset_from,
   const_raw_ptr_deref,
   const_raw_ptr_to_usize_cast,
+  const_saturating_int_methods,
   core_intrinsics,
   doc_cfg,
   exact_size_is_empty,
@@ -45,7 +53,9 @@ pub use crate::errors::{CapacityError, PushCapacityError};
 pub use crate::heap::{
   StaticHeap, StaticHeapDrainSorted, StaticHeapIntoIterSorted, StaticHeapPeekMut,
 };
-pub use crate::iterators::*;
+pub use crate::iterators::{
+  StaticVecDrain, StaticVecIntoIter, StaticVecIterConst, StaticVecIterMut,
+};
 pub use crate::string::{string_utils, StaticString, StringError};
 use crate::utils::{
   is_null_const, is_null_mut, quicksort_internal, reverse_copy, slice_from_raw_parts,
@@ -294,6 +304,10 @@ impl<T, const N: usize> StaticVec<T, N> {
   }
 
   /// Returns true if the current length of the StaticVec is greater than 0.
+  // Clippy wants `!is_empty()` for this, but I prefer it as-is. My question is though, does it
+  // actually know that we have an applicable `is_empty()` function, or is it just guessing? I'm not
+  // sure.
+  #[allow(clippy::len_zero)]
   #[inline(always)]
   pub const fn is_not_empty(&self) -> bool {
     self.length > 0
@@ -1070,10 +1084,12 @@ impl<T, const N: usize> StaticVec<T, N> {
       return StaticVec::new();
     }
     let mut res = StaticVec::new();
-    // The `as *mut T` cast here is necessary to make the type
-    // inference work properly (at the moment at least.) `rustc` still gets
-    // a bit confused by math operations done on const generic values
-    // in return types it seems.
+    // The `as *mut T` cast below is necessary to make the type inference work properly (at the
+    // moment at least). `rustc` still gets a bit confused by math operations done on const generic
+    // values in return types it seems.
+    // Note that the `StaticVec::new()` calls above *have* to be written without any constraints,
+    // as otherwise we'll hit a particular bug where `rustc` says:
+    // "expected struct `StaticVec<_, { N * 2 }>`, found struct `StaticVec<_, { N * 2 }>`".
     let mut res_ptr = res.as_mut_ptr() as *mut T;
     let mut i = 0;
     let length = self.length;
@@ -1082,8 +1098,8 @@ impl<T, const N: usize> StaticVec<T, N> {
         res_ptr.write(self.ptr_at_unchecked(i).read());
         res_ptr.offset(1).write(separator);
         res_ptr = res_ptr.offset(2);
-        i += 1
       }
+      i += 1;
     }
     unsafe {
       res_ptr.write(self.ptr_at_unchecked(i).read());
@@ -1171,6 +1187,20 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
   }
 
+  /// Inspired by the function of the same name from `ArrayVec`, this function directly returns
+  /// the StaticVec's backing array (as a "normal" array not wrapped in an instance of
+  /// `MaybeUninit`) in `Ok` if and only if the StaticVec is at maximum capacity. Otherwise, the
+  /// StaticVec itself is returned in `Err`.
+  #[inline(always)]
+  pub fn into_inner(mut self) -> Result<[T; N], Self> {
+    if self.length < N {
+      Err(self)
+    } else {
+      self.length = 0;
+      unsafe { Ok(self.data.read()) }
+    }
+  }
+
   /// Removes the specified range of elements from the StaticVec and returns them in a new one.
   #[inline]
   pub fn drain<R>(&mut self, range: R) -> Self
